import requests
import os
import subprocess
import shutil


def with_request(gist, headers):
    folder = name_folder(gist)
    if not os.path.exists(folder):
        os.mkdir(folder)
        print(f"Downloading the gist '{folder}'...")
    else:
        print(f"Updating the gist '{folder}'...")
        shutil.rmtree(folder)

    files = gist["files"]
    for filename in files:
        gist_url = files[filename]["raw_url"]
        response = requests.get(gist_url, headers=headers)
        with open(f"{folder}/{filename}", "wb") as file:
            file.write(response.content)


def with_git(gist):
    folder = name_folder(gist)
    gist_pull_url = f"git@gist.github.com:{gist['id']}.git"
    if not os.path.exists(folder):
        subprocess.call(["git", "clone", "--recursive", gist_pull_url, folder])
    else:
        subprocess.call(["git", "-C", folder, "pull", "--recurse-submodules"])


def name_folder(gist):
    return gist["description"] if gist["description"] != "" else gist["id"]


def download_gists(username, token, git_check):
    API_ENDPOINT = f"https://api.github.com/users/{username}/gists"

    headers = {
        "Authorization": f"token {token}",
    }

    if not token:
        headers = None

    response = requests.get(API_ENDPOINT, headers=headers)

    if response.status_code == 200:
        for gist in response.json():
            with_request(gist, headers) if not git_check else with_git(gist)
    else:
        print(response.status_code, response.text)



from .download import download_gists
from .environment import set_environment_variable
import argparse
import os


def main():
    parser = argparse.ArgumentParser(
        description="Download specified user's all gists at once"
    )
    parser.add_argument(
        "-u",
        "--username",
        type=str,
        metavar="Username",
        help="Github username to download gists from.",
    )
    parser.add_argument(
        "-t",
        "--token",
        type=str,
        metavar="Token",
        help="Github public access token if you want to also download private gists.",
    )
    parser.add_argument(
        "-e",
        "--environment",
        action="store_true",
        help="Whether to use environment variables or not.",
    )
    parser.add_argument(
        "-g",
        "--git",
        action="store_true",
        help="Whether to download with git or not. False by default since it's dependent on whether or not git is downloaded (and your ssh/gpg key). IF YOU TYPE ANYTHING IN AFTER -g/--git IT WILL BE ACCEPTED AS TRUE.",
    )
    parser.add_argument(
        "-gu",
        "--gusername",
        type=str,
        metavar="Username",
        help="Set Github username as environment variable.",
    )
    parser.add_argument(
        "-gpat",
        "--gpat",
        type=str,
        metavar="Token",
        help="Set Github personal access token as environment variable.",
    )

    args = parser.parse_args()

    if args.gusername:
        set_environment_variable("GITHUB_USERNAME", args.gusername)
    if args.gpat:
        set_environment_variable("GITHUB_PERSONAL_ACCESS_TOKEN", args.gpat)
    if not args.gusername and not args.gpat:
        if args.environment:  # false by default/if you don't use the argument
            username = os.getenv("GITHUB_USERNAME")
            token = os.getenv("GITHUB_PERSONAL_ACCESS_TOKEN")
        else:
            username = args.username
            token = args.token

        git_check = args.git  # false by default/if you don't use the argument

        if not username:
            print("Pass your Github username with -u.")
        else:
            download_gists(username, token, git_check)


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

VERSION = "1.18"
DESCRIPTION = "CLI for downloading all gists from a specified user."
with open("README.md", "r") as file:
    LONG_DESCRIPTION = file.read()
AUTHOR = "NecRaul"

setup(
    name="gist_neko",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    packages=find_packages(),
    install_requires=["requests"],
    keywords=[
        "python",
        "gists downloader",
        "downloader",
        "gists",
        "gist-neko",
        "kuroneko",
    ],
    url="https://github.com/NecRaul/gist-neko",
    project_urls={
        "Documentation": "https://github.com/NecRaul/gist-neko#readme",
        "Source": "https://github.com/NecRaul/gist-neko",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
    ],
    py_modules=["download", "environment"],
    entry_points={
        "console_scripts": [
            "gist-neko = gist_neko.main:main",
        ],
    },
)
